#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <time.h>
#include <string.h>

#ifndef GP2X
#include <SDL.h>
#endif

#ifdef GP2X
	#include <unistd.h>
	#include <fcntl.h>
	#include <sys/mman.h>
#endif

#define SCREEN_W	320
#define SCREEN_H	240

#define GPB_N		gpio[0]
#define GPB_NW		gpio[1]
#define GPB_W		gpio[2]
#define GPB_SW		gpio[3]
#define GPB_S		gpio[4]
#define GPB_SE		gpio[5]
#define GPB_E		gpio[6]
#define GPB_NE		gpio[7]
#define GPB_START	gpio[8]
#define GPB_SELECT	gpio[9]
#define GPB_L		gpio[10]
#define GPB_R		gpio[11]
#define GPB_A		gpio[12]
#define GPB_B		gpio[13]
#define GPB_X		gpio[14]
#define GPB_Y		gpio[15]
#define GPB_VOLP	gpio[16]
#define GPB_VOLM	gpio[17]
#define GPB_CLICK	gpio[18]

int8_t gpio[19];

#ifdef GP2X
	#define SYS_CLK_FREQ	7372800

	uint16_t *memregs16;
	uint32_t *memregs32;
	int32_t devgpio;
	int32_t devmem;
	int32_t devfb0;
	int32_t devfb1;

	uint32_t io_word, prev_io=0xFFFFFFFF;
#else
	SDL_Surface *screen;
	SDL_Event event;
	SDL_Joystick *joystick;

	int32_t prev_tick, tick, delay_ms;
#endif

// DATA
#include "title.c"
//#include "compo.c"
#include "coin.c"
#include "copter.c"
//#include "music.c"
#include "font.c"

#define RGB(x, y, z) ((x) << 11 | (y) << 6 | (z) << 1)	// RGB 5, 5, 5 -> RGB565 (I guess)

uint16_t *fb[2], *screen16;
int32_t sbuf;
int32_t a1=0, a2=0;
int32_t score=0;

#ifdef GP2X
void nsleep(uint32_t s, uint32_t ns)	//simplifies calls to nanosleep
{
	struct timespec tspec;

	tspec.tv_sec = (time_t) s;
	tspec.tv_nsec = ns;

	nanosleep(&tspec, NULL);
}

void io_parse(uint32_t io_word, uint32_t prev_io)
{
	uint32_t diff = prev_io ^ io_word;
	uint32_t i;

	for (i=0; i<19; i++)
		if ((diff>>i) & 1 == 1)
			gpio[i]	= (io_word>>i) & 1;

}

void gp_setFramebuffer(uint16_t *fbv, int vsync)
{
	uint32_t fb_addr;

	if (fbv== fb[0])
		fb_addr = 0x03101000;
	else
		fb_addr = 0x03381000;

	if (vsync)
	{
		while(memregs16[0x1182>>1]&(1<<4));	// waiting for V-sync
		while(!(memregs16[0x1182>>1]&(1<<4)));
	}

	memregs16[0x290E>>1] = (uint16_t)(fb_addr & 0xffff);
	memregs16[0x2910>>1] = (uint16_t)(fb_addr >> 16);
	memregs16[0x2912>>1] = (uint16_t)(fb_addr & 0xffff);
	memregs16[0x2914>>1] = (uint16_t)(fb_addr >> 16);
}
#else

void surf_array_o(SDL_Surface *surface, uint16_t *image, uint16_t width, uint16_t height)	//puts pixels from an array to a surface (optimized)
{	// the SDL_Surface and image are in 16-bit RGB565
	uint16_t *p;
 
	p = (uint16_t *) surface->pixels;		//p == address to the start pixel

	memcpy(p, image, height * width << 1);
}
#endif

void flip_fb()
{
	#ifdef GP2X
		gp_setFramebuffer(screen16, 1);
		sbuf=!sbuf;
		screen16 = fb[sbuf];
		//nsleep(0, 30*1000000);
	#else
		tick = SDL_GetTicks();
		delay_ms = 40 - (tick - prev_tick);
		if (delay_ms<0)
			delay_ms=0;
		prev_tick = tick;
		surf_array_o(screen, screen16, SCREEN_W, SCREEN_H);
		SDL_Flip(screen);
		SDL_Delay(delay_ms);
	#endif
}

inline void gp_drawPixel16(int x, int y, uint16_t c, uint16_t *fb)
{
	fb[y*SCREEN_W + x] = c;
}

void setpixel(int16_t x, int16_t y, uint16_t color, uint16_t *fb)
{
     if (x>=0 && x<SCREEN_W && y>=0 && y<SCREEN_H)
	     gp_drawPixel16(x, y, color, fb);
}

void gp_clearFramebuffer16(uint16_t *fb, uint8_t whocaresitsalwayszero)
{
	memset(fb, 0, SCREEN_W*SCREEN_H*2);
}

void gp_drawSpriteT(uint16_t *sprite, short put_x, short put_y, uint16_t *fb, uint16_t trans, uint16_t xsize, uint16_t ysize)
{
	int16_t xx, yy;
	int32_t i=0;
	uint16_t color;

	for (yy=0; yy<ysize; yy++)
		for (xx=0; xx<xsize; xx++)
		{
			color = sprite[i++];
			if (color != trans)
				setpixel(put_x+xx, put_y+yy, color, fb);
		}
}

void draw_line(int32_t x1, int32_t y1, int32_t x2, int32_t y2, uint16_t color, uint16_t *fb)
{
	int32_t deltaRow, deltaCol;
	int32_t stepRow, stepCol;
	int32_t errorRow, errorCol;
	int32_t detdelta;
	int32_t count;

	int32_t rr1, rr2, cc1, cc2;

	rr1 = x1;
	rr2 = x2;
	cc1 = y1;
	cc2 = y2;

	/* horizontal line */
	if (rr1 == rr2)
	{
		if (cc1 > cc2)
			stepCol = -1;
		else
			stepCol = 1;

		for (; cc1 != cc2; cc1 += stepCol)
		{
			if (cc1>=0 && cc1<SCREEN_H && rr1>=0 && rr1<SCREEN_W)
				fb[cc1 * SCREEN_W + rr1] = color;
		}

		if (cc1>=0 && cc1<SCREEN_H && rr1>=0 && rr1<SCREEN_W)
			fb[cc1 * SCREEN_W + rr1] = color;
		return;
	}


	/* vertical line */
	if (cc1 == cc2)
	{
		if (rr1 > rr2)
			stepRow = -1;
		else
			stepRow = 1;

		for (; rr1 != rr2; rr1 += stepRow)
		{
			if (cc1>=0 && cc1<SCREEN_H && rr1>=0 && rr1<SCREEN_W)
				fb[cc1 * SCREEN_W + rr1] = color;
		}

		if (cc1>=0 && cc1<SCREEN_H && rr1>=0 && rr1<SCREEN_W)
			fb[cc1 * SCREEN_W + rr1] = color;
		return;
	}


	if (cc1>=0 && cc1<SCREEN_H && rr1>=0 && rr1<SCREEN_W)
		fb[cc1 * SCREEN_W + rr1] = color;

	stepCol = 1;
	deltaCol = cc2-cc1;
	if (deltaCol < 0)
	{
		stepCol = -1;
		deltaCol = -deltaCol;
	}

	stepRow = 1;
	deltaRow = rr2 - rr1;
	if (deltaRow < 0)
	{
		stepRow = -1;
		deltaRow = -deltaRow;
	}

	if (deltaCol > deltaRow)
	{
		count=deltaCol;
		detdelta=deltaCol;
		errorRow=deltaCol >> 1;
		errorCol=0;
	}
	else
	{
		count=deltaRow;
		detdelta=deltaRow;
		errorRow=0;
		errorCol=deltaRow >> 1;
	}

	do
	{
		errorCol=(errorCol+deltaCol);
		if (errorCol>=detdelta)
		{
			errorCol-=detdelta;
			cc1 += stepCol;
		}

		errorRow=(errorRow+deltaRow);
		if (errorRow>=detdelta)
		{
			errorRow-=detdelta;
			rr1 += stepRow;
		}

		if (cc1>=0 && cc1<SCREEN_H && rr1>=0 && rr1<SCREEN_W)
			fb[cc1 * SCREEN_W + rr1] = color;

		count--;
	}
	while ((int32_t)count>0);

	if (cc1>=0 && cc1<SCREEN_H && rr1>=0 && rr1<SCREEN_W)
		fb[cc1 * SCREEN_W + rr1] = color;

}

void gp_drawLine16(int x0, int y0, int x1, int y1, uint16_t color, uint16_t *fb)
{
	int dy = y1 - y0;
	int dx = x1 - x0;
	int stepx, stepy;

	if (dy < 0)
	{
		dy = -dy;
		stepy = -1;
	}
	else
	{
		stepy = 1;
	}
	if (dx < 0)
	{
		dx = -dx;
		stepx = -1;
	}
	else
	{
		stepx = 1;
	}
	dy <<= 1;
	dx <<= 1;

	setpixel(x0, y0, color, fb);
	if (dx > dy)
	{
		int fraction = dy - (dx >> 1);
		while (x0 != x1)
		{
			if (fraction >= 0)
			{
				y0 += stepy;
				fraction -= dx;
			}
			x0 += stepx;
			fraction += dy;
			setpixel(x0, y0, color, fb);
		}
	}
	else
	{
		int fraction = dx - (dy >> 1);
		while (y0 != y1)
		{
			if (fraction >= 0)
			{
				x0 += stepx;
				fraction -= dy;
			}
			y0 += stepy;
			fraction += dx;
			setpixel(x0, y0, color, fb);
		}
	}
}

void drawchar(uint16_t *sprite, int32_t x, int32_t y, uint16_t color, int32_t chr, uint16_t *fb)
{
	int32_t xx, yy, xpos, ypos;
	uint16_t pixel;

	xpos=(chr % 32)*8;
	ypos=(chr / 32)*16;

	for (yy=0;yy<16;yy++)
	{
		for (xx=0;xx<8;xx++)
		{
			pixel=sprite[xpos+ypos*256];
			if (xx+x<SCREEN_W && xx+x>0)
				if (pixel==0) gp_drawPixel16(xx+x, yy+y, color, fb);
			xpos++;
		}
		xpos=xpos-8;
		ypos++;
	}
}

void text(int32_t x, int32_t y, uint16_t color, char *buffer, uint16_t *fb)
{
	int32_t i, len, c;
	len=strlen(buffer);
	for (i=0;i<len;i++)
	{
		c=buffer[i];
		drawchar((uint16_t *)font, x+i*8, y, color, c, fb);
	}
}

void box(int32_t x, int32_t y, int32_t xx, int32_t yy, uint16_t color, uint16_t *fb)
{
	int32_t i;
	for (i=y;i<=yy;i++)
	{
		gp_drawLine16(x, i, xx, i, color, fb);
	}
}

void drawcave(int32_t x, int32_t y, int32_t size, uint16_t *fb)
{
	int32_t xx=x+9;
	if (xx>SCREEN_W-1) xx=SCREEN_W;
	if (x<0) x=0;
	box(x, 0, xx, y-1, RGB(0, 28, 0), fb);
//	box(x, y, xx, y+size, 0x00, fb);
	box(x, y+size+1, xx, SCREEN_W-1, RGB(0, 28, 0), fb);

}

uint16_t px(int32_t x, int32_t y, uint16_t *fb)
{
	uint16_t col;
	col=fb[y*SCREEN_W + x];
	return(col);
}

void redbar(int32_t y, uint16_t *fb)
{
	uint16_t col;
	int32_t i;
	int32_t cnt=0;

	for (i=0;i<=7;i++)
	{
		col=RGB(i*4, 0, 0);
		gp_drawLine16(0, y+cnt-1, SCREEN_W, y+cnt, col, fb);
		cnt++;
	}
	for (i=7;i>=0;i--)
	{
		col=RGB(i*4, 0, 0);
		gp_drawLine16(0, y+cnt-1, SCREEN_W, y+cnt, col, fb);
		cnt++;
	}
}

int32_t title(void)
{
	int32_t i;
	int32_t cvy[34];
	int32_t cvs[34];
	int32_t scr=0;
	int32_t chscr=0;
	int32_t scrpos=0;
	int32_t o;
	int32_t menu=2;
	uint16_t gelb=RGB(31, 31, 0);
	uint16_t norm=RGB(16, 16, 16);
	uint16_t colz;
	int32_t gmmode=0;
	int32_t gmspd=0;
	int32_t penalty=10;
	int32_t nofire=0;
	uint8_t scroller[]="                                        Cave Copter! Ported by A_SN, who deserves a medal for not surrendering to the awfulness of the code he had to attempt to improve                                                                                                                                                          ";

	gmmode=a1;
	gmspd=a2;
	char xscr[40];
	for (i=0;i<=33;i++)
	{
		cvy[i]=30;
		cvs[i]=180;
	}
	while (nofire==0)
	{
		gp_clearFramebuffer16(screen16, 0x00);
		for (i=0;i<=33;i++)
		{
			drawcave(i*10-scr, cvy[i], cvs[i], screen16);
		}
		scr+=2;

		if (scr==10)
		{
			scr=0;
			for (i=1;i<=33;i++)
			{
				cvy[i-1]=cvy[i];
				cvs[i-1]=cvs[i];
			}
			if (rand()%2==0) cvy[33]-=4;
			else cvy[33]+=4;

		}
		gp_drawSpriteT((uint16_t *)titlegfx, 10, 10, screen16, 0xFFFE, 300, 78);
		o=0;
		for (i=scrpos;i<=scrpos+40;i++)
		{
			drawchar((uint16_t *)font, o*8-chscr-1, 220, 0x00, scroller[i], screen16);
			drawchar((uint16_t *)font, o*8-chscr+1, 220, 0x00, scroller[i], screen16);
			drawchar((uint16_t *)font, o*8-chscr, 219, 0x00, scroller[i], screen16);
			drawchar((uint16_t *)font, o*8-chscr-1, 221, 0x00, scroller[i], screen16);
			drawchar((uint16_t *)font, o*8-chscr, 220, 0xFFFF, scroller[i], screen16);
			o++;
		}

		sprintf(xscr, "LAST SCORE: %i", score);
		text(100, 180, gelb, xscr, screen16);
		if (menu==0) redbar(100, screen16);
		if (menu==1) redbar(130, screen16);
		if (menu==2) redbar(160, screen16);
		text(35, 100, 0xFFFF, "GAME MODE -", screen16);
		if (gmmode==0) colz=gelb;
		else colz=norm;
		text(139, 100, colz, "NORMAL", screen16);
		if (gmmode==1) colz=gelb;
		else colz=norm;
		text(195, 100, colz, "COIN", screen16);
		if (gmmode==2) colz=gelb;
		else colz=norm;
		text(235, 100, colz, "MOVING", screen16);
		text(27, 130, 0xFFFF, "SPEED -", screen16);
		if (gmspd==0) colz=gelb;
		else colz=norm;
		text(91, 130, colz, "SLOW", screen16);
		if (gmspd==1) colz=gelb;
		else colz=norm;
		text(131, 130, colz, "MEDIUM", screen16);
		if (gmspd==2) colz=gelb;
		else colz=norm;
		text(187, 130, colz, "FAST", screen16);
		if (gmspd==3) colz=gelb;
		else colz=norm;
		text(227, 130, colz, "MADNESS!", screen16);

		text(120, 160, 0xFFFF, "LET'S GO!", screen16);
		if (penalty==0)
		{
			//****Input handling****
			#ifdef GP2X
			read(devgpio, &io_word, 4);
			io_word &= 0x7FFFF;
			if (io_word != prev_io)
			{
				io_parse(io_word, prev_io);
				prev_io = io_word;
			}
			#else

			if (SDL_PollEvent(&event))
			{
				if (event.type==SDL_KEYDOWN)
					switch (event.key.keysym.sym)
					{
						case SDLK_UP:
							GPB_N=1;
							break;
						case SDLK_DOWN:
							GPB_S=1;
							break;
						case SDLK_LEFT:
							GPB_W=1;
							break;
						case SDLK_RIGHT:
							GPB_E=1;
							break;
						case SDLK_SPACE:
							GPB_A=1;
							break;
					}
				if (event.type==SDL_KEYUP)
					switch (event.key.keysym.sym)
					{
						case SDLK_UP:
							GPB_N=0;
							break;
						case SDLK_DOWN:
							GPB_S=0;
							break;
						case SDLK_LEFT:
							GPB_W=0;
							break;
						case SDLK_RIGHT:
							GPB_E=0;
							break;
						case SDLK_SPACE:
							GPB_A=0;
							break;
					}
			}
			#endif
			//----Input handling----

			if (GPB_S && menu<2)
			{
				menu++;
				penalty=5;
			}
			if (GPB_N && menu>0)
			{
				menu--;
				penalty=5;
			}
			if (menu==0)
			{
				if (GPB_E && gmmode<2)
				{
					gmmode++;
					penalty=5;
				}
				if (GPB_W && gmmode>0)
				{
					gmmode--;
					penalty=5;
				}
			}
			if (menu==1)
			{
				if (GPB_E && gmspd<3)
				{
					gmspd++;
					penalty=5;
				}
				if (GPB_W && gmspd>0)
				{
					gmspd--;
					penalty=5;
				}
			}
			if (menu==2)
			{
				if (GPB_A)
				{
					nofire=1;
				}
			}
		}

		flip_fb();
		chscr++;
		if (chscr==8)
		{
			chscr=0;
			scrpos++;
			if (scrpos==700) scrpos=0;
		}
		
		if (penalty>0) penalty--;
	}

	return(gmmode*10+gmspd);
}

void game(int32_t gm)
{
	int32_t i;
	int32_t cvy[34];
	int32_t scr=0;
	int32_t cvs[34];
	int32_t blockx=-30, blocky;
	int32_t speed=5;
	int32_t helix=50, heliy=109;
	int32_t drop=0;
	int32_t thrust=0;
	int32_t counters=0;
	int32_t counterup=0;
	int32_t lift=0;
	int32_t divider=80;
	int32_t mode=1;
	int32_t move=1;
	int32_t fcount=0;
	//int32_t a1, a2;
	int32_t coinx=-40, coiny;
	int32_t hit=0;
	char xscr[20];
	// clear cave
	score=0;
	a1=gm/10;
	a2=gm%10;
	mode=a1;
	if (a2==0) speed=1;
	if (a2==1) speed=2;
	if (a2==2) speed=5;
	if (a2==3) speed=10;
	for (i=0;i<=33;i++)
	{
		cvy[i]=30;
		cvs[i]=180;
	}


	while (hit!=1)
	{
		fcount++;
		counters++;
		gp_clearFramebuffer16(screen16, 0x00);
		for (i=0;i<=33;i++)
		{
			drawcave(i*10-scr, cvy[i], cvs[i], screen16);
		}
		scr+=speed;
		if (scr==10)
		{
			scr=0;
			for (i=1;i<=33;i++)
			{
				cvy[i-1]=cvy[i];
				cvs[i-1]=cvs[i];
			}
			if (rand()%2==0)
			{
				if (cvy[33]>5)
				{
					cvy[33]-=4;
					cvs[33]=180-fcount/100;
				}
			}
			else
			{
				if (cvy[33]<55)
				{
					cvy[33]+=4;
					cvs[33]=180-fcount/100;
				}

			}
		}
		if (blockx<-20) 	// spawn block?
		{
			blockx=SCREEN_W; 	// block spawned
			blocky=rand()%180;
			move=rand()%2;
		}
		if (blockx>-30)		// draw block
		{
			box(blockx, blocky, blockx+19, blocky+59, RGB(0, 28, 0), screen16);
			blockx-=speed;
			if (mode==2)
			{
				if (move==1)
				{
					if (blocky<180) blocky++;
					if (blocky>=180) move=0;
				}
				if (move==0)
				{
					if (blocky>0) blocky--;
					if (blocky<=0) move=1;
				}
			}
		}
		if (mode==1)
		{
			// coin mode

			if (coinx==-40)
			{
				if (rand()%80==0 && blockx<300)
				{
					coinx=SCREEN_W;
					coiny=rand()%121+60;
				}
			}

			if (coinx>-40)
			{
				gp_drawSpriteT((uint16_t *)coin, coinx, coiny, screen16, 0xFFFE, 32, 32);
				coinx-=speed;
			}
			if (px(helix-1, heliy-1, screen16)==0xF740 ||
			        px(helix+10, heliy+1, screen16)==0xF740 ||
			        px(helix+31, heliy-1, screen16)==0xF740 ||
			        px(helix+50, heliy+2, screen16)==0xF740 ||
			        px(helix+42, heliy+13, screen16)==0xF740 ||
			        px(helix+42, heliy+22, screen16)==0xF740 ||
			        px(helix+32, heliy+23, screen16)==0xF740 ||
			        px(helix+22, heliy+22, screen16)==0xF740 ||
			        px(helix+15, heliy+13, screen16)==0xF740 ||
			        px(helix+3, heliy+17, screen16)==0xF740)
			{
				score+=40*speed;
				coinx=-40;
			}
		}


		// draw helicopter
		if (counters<=3) gp_drawSpriteT((uint16_t *)copter0, helix, heliy, screen16, 0xFFFE, 50, 23);
		if (counters>3) gp_drawSpriteT((uint16_t *)copter1, helix, heliy, screen16, 0xFFFE, 50, 23);
		if (counters==6) counters=0;

		//collision?

		if (px(helix-1, heliy-1, screen16)==RGB(0, 28, 0) ||
		        px(helix+10, heliy+1, screen16)==RGB(0, 28, 0) ||
		        px(helix+31, heliy-1, screen16)==RGB(0, 28, 0) ||
		        px(helix+50, heliy+2, screen16)==RGB(0, 28, 0) ||
		        px(helix+42, heliy+13, screen16)==RGB(0, 28, 0) ||
		        px(helix+42, heliy+22, screen16)==RGB(0, 28, 0) ||
		        px(helix+32, heliy+23, screen16)==RGB(0, 28, 0) ||
		        px(helix+22, heliy+22, screen16)==RGB(0, 28, 0) ||
		        px(helix+15, heliy+13, screen16)==RGB(0, 28, 0) ||
		        px(helix+3, heliy+17, screen16)==RGB(0, 28, 0))
		{
			hit=1;
		}
		//drop?


		//****Input handling****
		#ifdef GP2X
		read(devgpio, &io_word, 4);
		io_word &= 0x7FFFF;
		if (io_word != prev_io)
		{
			io_parse(io_word, prev_io);
			prev_io = io_word;
		}
		#else

		if (SDL_PollEvent(&event))
		{
			if (event.type==SDL_KEYDOWN)
				switch (event.key.keysym.sym)
				{
					case SDLK_SPACE:
						GPB_A=1;
						break;
				}
			if (event.type==SDL_KEYUP)
				switch (event.key.keysym.sym)
				{
					case SDLK_SPACE:
						GPB_A=0;
						break;
				}
		}
		#endif
		//----Input handling----
	
		if (GPB_A) drop--;
		else drop++;
		if (drop<0) lift=-((drop-9)*(drop-9)/divider);
		if (drop>=0) lift=(drop+9)*(drop+9)/divider;


		sprintf(xscr, "SCORE: %i", score);
		text(10, 220, RGB(31, 31, 0), xscr, screen16);

		heliy+=lift;
		score+=speed;

		flip_fb();
	}
}

int main(int argc, char *argv[])
{
	int32_t x;
	/*fb[0] = (uint16_t*) FRAMEBUFFER1;  // 0x0C7B4000
	fb[1] = (uint16_t*) FRAMEBUFFER2;  // 0x0C7DA000
	gp_setMMU(FRAMEBUFFER1, FRAMEBUFFER2+(SCREEN_W*SCREEN_H*2)-1, 0xFF2);*/	// TODO replace with mmap
	/*gp_startSoundmixer(0);	// TODO figure out how to do that
	gp_startModfile(music);*/
	/*gp_initFramebuffer(fb[0], 16, 60);
	gp_clearFramebuffer16(fb[0], 0x00);
	gp_clearFramebuffer16(fb[1], 0x00);
	gp_setFramebuffer(fb[0], 1);*/

	#ifdef GP2X		
		devgpio = open("/dev/GPIO", O_RDONLY);
		devmem = open("/dev/mem", O_RDWR);
		devfb0 = open("/dev/fb0", O_RDWR);
		devfb1 = open("/dev/fb1", O_RDWR);

		memregs32 = (uint32_t *) mmap(0, 0x10000, PROT_READ|PROT_WRITE, MAP_SHARED, devmem, 0xc0000000);
		memregs16 = (uint16_t *) memregs32;
		fb[0] = (uint16_t *) mmap(0, SCREEN_W*SCREEN_H*sizeof(uint16_t), PROT_WRITE, MAP_SHARED, devfb0, 0);
		if (fb[0] == NULL)
			fprintf(stderr, "malloc returned null on fb0\n");
		fb[1] = (uint16_t *) mmap(0, SCREEN_W*SCREEN_H*sizeof(uint16_t), PROT_WRITE, MAP_SHARED, devfb1, 0);
		if (fb[1] == NULL)
			fprintf(stderr, "malloc returned null on fb1\n");

		sbuf = 0;
		screen16 = fb[sbuf];
	#else
		SDL_Init(SDL_INIT_EVERYTHING);
		atexit(SDL_Quit);
		joystick = SDL_JoystickOpen(0);
		screen=SDL_SetVideoMode(SCREEN_W, SCREEN_H, 16, SDL_HWSURFACE | SDL_DOUBLEBUF);
		screen16 = calloc (SCREEN_W * SCREEN_H, sizeof(uint16_t));

		prev_tick=SDL_GetTicks();
	#endif

	while (1)
	{
		x=title();
		game(x);
	}

	return 0;
}
